#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025

"""Module containing the SDC API client."""

import json
import requests
import urllib3
from ibm_watsonx_data_integration.common.utils import wait_and_retry_on_http_error
from ibm_watsonx_data_integration.cpd_api.base import BaseAPIClient
from typing import TYPE_CHECKING, Any
from urllib.parse import quote

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.common.auth import BaseAuthenticator

DEFAULT_SDC_API_VERSION = 1
REQUIRED_HEADERS = {"X-Requested-By": "sdc", "X-SS-REST-CALL": "true", "content-type": "application/json"}


class DataCollectorAPIClient(BaseAPIClient):
    """The API Client of the Data Collector."""

    def __init__(
        self,
        auth: "BaseAuthenticator",  # pragma: allowlist secret
        engine_url: str,
    ) -> None:
        """The __init__ of the EngineApiClient.

        Args:
            auth: The Authentication object.
            engine_url: The URL of the engine.
        """
        super().__init__(auth=auth, base_url=engine_url)
        self.url = f"{self.base_url}/rest/v{DEFAULT_SDC_API_VERSION}"

        # As of now, sdc uses a self-signed certificate from the machine it is running in.
        # It is impossible to make calls to the sdc without being on that machine ourselves,
        # this will be a known issue till we have tunneling set up.
        # requests made through this class will make it requests without verification
        # ToDo: WSDK-210
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    @wait_and_retry_on_http_error(timeout_sec=10)
    def create_pipeline(self, pipeline_title: str, params: dict[str, Any] | None = None) -> requests.Response:
        """Create a pipeline in a Datacollector."""
        url = f"{self.url}/pipeline/{quote(pipeline_title, safe='')}"
        return self.put(url=url, params=params, headers=REQUIRED_HEADERS, verify=False)

    @wait_and_retry_on_http_error(timeout_sec=10)
    def update_pipeline(self, pipeline_id: str, pipeline_definition: dict) -> requests.Response:
        """Update a pipeline in a Datacollector."""
        url = f"{self.url}/pipeline/{quote(pipeline_id, safe='')}"
        return self.post(url=url, data=json.dumps(pipeline_definition), headers=REQUIRED_HEADERS, verify=False)

    @wait_and_retry_on_http_error(timeout_sec=10)
    def get_pipeline_by_id(self, pipeline_id: str) -> requests.Response:
        """Get a pipeline in a DataCollector."""
        url = f"{self.url}/pipeline/{quote(pipeline_id, safe='')}"
        return self.get(url=url, headers=REQUIRED_HEADERS, verify=False)

    @wait_and_retry_on_http_error(timeout_sec=10)
    def delete_pipeline(self, pipeline_id: str) -> requests.Response:
        """Delete a pipeline in a Datacollector."""
        url = f"{self.url}/pipeline/{quote(pipeline_id, safe='')}"
        return self.delete(url=url, headers=REQUIRED_HEADERS, verify=False)

    @wait_and_retry_on_http_error(timeout_sec=10)
    def validate_pipeline(self, pipeline_id: str) -> requests.Response:
        """Validate a pipeline in a DataCollector."""
        url = f"{self.url}/pipeline/{quote(pipeline_id, safe='')}/validate"
        response = self.get(url=url, headers=REQUIRED_HEADERS, verify=False)
        return response

    @wait_and_retry_on_http_error(timeout_sec=10)
    def create_pipeline_preview(self, pipeline_id: str) -> requests.Response:
        """Creates a pipeline preview on the engine."""
        url = f"{self.url}/pipeline/{quote(pipeline_id, safe='')}/preview"
        return self.post(
            url=url,
            headers=REQUIRED_HEADERS,
            verify=False,
            params=dict(
                batchSize="10",
                skipTargets="true",
                timeout="120000",
                skipLifecycleEvents="true",
                testOrigin="false",
                pushLimitDown="true",
                remote="true",
            ),
        )

    @wait_and_retry_on_http_error(timeout_sec=10)
    def get_pipeline_preview_status(self, pipeline_id: str, previewer_id: str) -> requests.Response:
        """Get a pipeline status in a DataCollector."""
        url = f"{self.url}/pipeline/{quote(pipeline_id, safe='')}/preview/{quote(previewer_id, safe='')}/status"
        response = self.get(url=url, headers=REQUIRED_HEADERS, verify=False)
        return response

    @wait_and_retry_on_http_error(timeout_sec=10)
    def get_pipeline_preview(self, pipeline_id: str, previewer_id: str) -> requests.Response:
        """Get a pipeline status in a DataCollector."""
        url = f"{self.url}/pipeline/{quote(pipeline_id, safe='')}/preview/{quote(previewer_id, safe='')}"
        response = self.get(url=url, headers=REQUIRED_HEADERS, verify=False)
        return response

    @wait_and_retry_on_http_error(timeout_sec=10)
    def get_pipeline_validation_status(self, pipeline_id: str, data: dict) -> requests.Response:
        """A final step of validating a pipeline in a DataCollector. It returns all the errors."""
        url = f"{self.url}/pipeline/{quote(pipeline_id, safe='')}"
        response = self.post(url=url, data=json.dumps(data), headers=REQUIRED_HEADERS, verify=False)
        return response

    @wait_and_retry_on_http_error(timeout_sec=10)
    def get_library_definitions(self) -> requests.Response:
        """Get the library definitions of an engine."""
        url = f"{self.url}/definitions"
        return self.get(url=url, verify=False, params=dict(schemaVersion=2))



from ibm_watsonx_data_integration.services.datastage.models.enums import AZURESQL_CONNECTION
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAzureSqlConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "azuresql_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _mytesttable_1_ = fc.add_stage("Microsoft Azure SQL Database", "mytesttable_1")
        _mytesttable_1_.configuration.schema_name = "tm_ds"
        _mytesttable_1_.configuration.table_name = "mytesttable"

        _mytesttable_1_.configuration.connection.authentication_method = AZURESQL_CONNECTION.AuthMethod.entra_id_service
        _mytesttable_1_.configuration.connection.client_id = environ["AZURESQL_CLIENT_ID"]
        _mytesttable_1_.configuration.connection.client_secret = environ["AZURESQL_CLIENT_SECRET"]
        _mytesttable_1_.configuration.connection.database = "CONOPSDB"
        _mytesttable_1_.configuration.connection.hostname_or_ip_address = environ["AZURESQL_HOST"]
        _mytesttable_1_.configuration.connection.port = "1433"
        _mytesttable_1_.configuration.connection.proxy = False
        _mytesttable_1_.configuration.connection.port_is_ssl_enabled = True

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        mytesttable_1_schema = _mytesttable_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        mytesttable_1_schema.add_field("CHAR", "col1").nullable().length(6)
        mytesttable_1_schema.add_field("DECIMAL", "col2").nullable().length(6).precision(6)
        mytesttable_1_schema.add_field("VARCHAR", "col3").nullable().length(6)
        mytesttable_1_schema.add_field("FLOAT", "col4").nullable()
        mytesttable_1_schema.add_field("BIGINT", "col6").nullable()
        mytesttable_1_schema.add_field("BINARY", "col7").nullable().length(6)
        mytesttable_1_schema.add_field("FLOAT", "col8").nullable()
        mytesttable_1_schema.add_field("BIT", "col9").nullable()
        mytesttable_1_schema.add_field("DATE", "col10").nullable()
        mytesttable_1_schema.add_field("INTEGER", "col11").nullable()
        mytesttable_1_schema.add_field("VARCHAR", "col12").nullable().length(4000)
        mytesttable_1_schema.add_field("VARCHAR", "col13").nullable().length(8000)
        mytesttable_1_schema.add_field("VARBINARY", "col14").nullable().length(8000)
        mytesttable_1_schema.add_field("CHAR", "col15").nullable().length(6)
        mytesttable_1_schema.add_field("NUMERIC", "col16").nullable().length(6)
        mytesttable_1_schema.add_field("VARCHAR", "col17").nullable().length(6)
        mytesttable_1_schema.add_field("REAL", "col18").nullable().length(7)
        mytesttable_1_schema.add_field("SMALLINT", "col19").nullable()
        mytesttable_1_schema.add_field("TIMESTAMP", "col20").nullable()
        mytesttable_1_schema.add_field("TIMESTAMP", "col21").nullable()
        mytesttable_1_schema.add_field("TINYINT", "col22").nullable().unsigned()
        mytesttable_1_schema.add_field("VARBINARY", "col23").nullable().length(6)
        mytesttable_1_schema.add_field("VARCHAR", "col24").nullable().length(6)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from ibm_watsonx_data_integration.services.datastage.models.enums import DB2FORDATASTAGE_CONNECTION
from ibm_watsonx_data_integration.services.datastage.models.flow.datastage_flow import DataStageFlow
from ibm_watsonx_data_integration.services.datastage.tests.integration.util import AbstractFlowRunTest
from os import environ
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestDb2DatastageConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "db2datastage_peek"

    def get_fc(self, project: Project):
        fc = DataStageFlow(project=project, name=self.get_flow_name(), description="", flow_type="datastage")

        # Stages
        _autotesttbl_1_ = fc.add_stage("IBM Db2 for DataStage", "AUTOTESTTBL_1")
        _autotesttbl_1_.configuration.generate_sql_at_runtime = True
        _autotesttbl_1_.configuration.table_name = '"TESTUSER"."AUTOTESTTBL"'
        _autotesttbl_1_.configuration.enable_quoted_identifiers = True

        _autotesttbl_1_.configuration.connection.hostname = environ["DB2DATASTAGE_ADVANCED_HOST"]
        _autotesttbl_1_.configuration.connection.ssl_connection = False
        _autotesttbl_1_.configuration.connection.credentials = (
            DB2FORDATASTAGE_CONNECTION.AuthenticationType.username_and_password
        )
        _autotesttbl_1_.configuration.connection.cas_lite_service_authorization_header = ""
        _autotesttbl_1_.configuration.connection.database = "CONOPS"
        _autotesttbl_1_.configuration.connection.keep_conductor_connection_alive = False
        _autotesttbl_1_.configuration.connection.password = environ["DB2DATASTAGE_PASSWORD"]
        _autotesttbl_1_.configuration.connection.username = environ["DB2DATASTAGE_USERNAME"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        autotesttbl_1_schema = _autotesttbl_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        autotesttbl_1_schema.add_field("CHAR", "COL1").nullable().length(6)
        autotesttbl_1_schema.add_field("DECIMAL", "COL2").nullable().length(6).precision(6)
        autotesttbl_1_schema.add_field("VARCHAR", "COL3").nullable().length(6)
        autotesttbl_1_schema.add_field("DOUBLE", "COL4").nullable()
        autotesttbl_1_schema.add_field("BIGINT", "COL6").nullable()
        autotesttbl_1_schema.add_field("REAL", "COL8").nullable().length(4)
        autotesttbl_1_schema.add_field("DATE", "COL10").nullable()
        autotesttbl_1_schema.add_field("INTEGER", "COL11").nullable()
        autotesttbl_1_schema.add_field("SMALLINT", "COL19").nullable()
        autotesttbl_1_schema.add_field("TIME", "COL20").nullable()

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025


"""This module contains the AccountAPIClient class."""

import requests
from ibm_watsonx_data_integration.cpd_api.base import BaseAPIClient
from typing import TYPE_CHECKING
from urllib.parse import quote

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.common.auth import BaseAuthenticator

DEFAULT_ROLE_API_VERSION = 2


class RoleAPIClient(BaseAPIClient):
    """The API client of the Role."""

    def __init__(
        self,
        auth: "BaseAuthenticator",  # pragma: allowlist secret
        base_url: str = "https://iam.cloud.ibm.com",
    ) -> None:
        """Initializes the RoleAPIClient.

        Args:
            auth: The Authentication object.
            base_url: The Role Service URL.
        """
        super().__init__(auth=auth, base_url=base_url)
        self.url_path = f"v{DEFAULT_ROLE_API_VERSION}/roles"

    def get_roles(
        self,
        params: dict | None = None,
    ) -> requests.Response:
        """Get account owner details for a given account ID.

        Args:
            params: Params to filter by. This can contain: account_id, service_name, source_service_name, policy_type,
                    service_group_id.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}"
        return self.get(url=url, params=params)

    def retrieve_role(self, role_id: str) -> requests.Response:
        """Retrieve a role by providing a role ID.

        Args:
            role_id: The role id.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(role_id, safe='')}"
        return self.get(url=url)

    def create_role(self, data: str) -> requests.Response:
        """Creates a custom role for a specific service within the account.

        Args:
            data: Data to create a role. this should contain: name, display_name, description, account_id,
                  service_name, actions.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}"
        response = self.post(url=url, data=data)
        return response

    def update_role(self, role_id: str, etag: str, data: str) -> requests.Response:
        """Updates a custom role.

        Args:
            role_id: The role id.
            etag: The revision number for updating a policy and must match the Etag value of
            the existing policy. The Etag can be retrieved using the GET /v2/policies/{id} API and looking
            at the Etag response header.
            data: Data to update a role. Possible fields are: display_name, description, actions.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(role_id, safe='')}"
        headers = {"If-Match": etag}
        response = self.put(url=url, headers=headers, data=data)
        return response

    def delete_role(self, role_id: str) -> requests.Response:
        """Delete a role by providing a role ID.

        Args:
            role_id: The role id.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(role_id, safe='')}"
        response = self.delete(url=url)
        return response



import ibm_watsonx_data_integration.services.datastage.models.flow_json_model as models
import json
from abc import ABC, abstractmethod
from ibm_watsonx_data_integration.services.datastage.models.enums import FIELD
from typing import TypeVar
T = TypeVar("T", bound="BaseField")

class BaseField(ABC):
    def __init__(self, name: str, configuration: models.FieldModelComplex = None):
        if configuration:
            self.configuration = configuration
            self.configuration.name = name
        else:
            default_meta = models.Metadata(
                is_key=False,
                item_index=0,
                is_signed=True,
                description="",
                min_length=0,
                decimal_precision=100,
                decimal_scale=0,
            )
            self.configuration = models.FieldModelComplex(
                name=name,
                type="string",
                odbc_type="CHAR",
                metadata=default_meta,
                nullable=False,
                app_data={"time_scale": 0},
            )

    def __str__(self):
        dictionary = {}
        dictionary["name"] = self.configuration.name
        dictionary["type"] = self.configuration.type
        if self.configuration.metadata:
            dictionary["metadata"] = json.loads(str(self.configuration.metadata))
        dictionary["nullable"] = self.configuration.nullable
        dictionary["app_data"] = self.configuration.app_data

        return json.dumps(dictionary, indent=4)

    @abstractmethod
    def _create_from_model(self, model: models.FieldModelComplex)-> T: ...

    def clone(self):
        model = self.configuration.model_copy(deep=True)
        return self._create_from_model(model)

    def _set_model_type(self, typ: str):
        self.configuration.type = typ
        return self

    def _set_app_type_code(self, typ_code: str):
        self.configuration.app_data["type_code"] = typ_code
        return self

    def _set_app_odbc_type(self, odbc_typ: str):
        self.configuration.app_data["odbc_type"] = odbc_typ
        self.configuration.odbc_type = odbc_typ

    def name(self, name: str)-> T:
        self.configuration.name = name
        return self

    def nullable(self, is_nullable: bool = True):
        self.configuration.nullable = is_nullable
        return self

    def key(self, is_key: bool = True):
        self.configuration.metadata.is_key = is_key
        self.configuration.primary_key = True
        return self

    def source(self, name: str):
        self.configuration.metadata.source_field_id = name
        return self

    def description(self, description: str):
        self.configuration.metadata.description = description
        self.configuration.description = description
        return self

    def pivot(self, pivot_property: str = None):
        self.configuration.app_data["pivot_property"] = pivot_property
        return self

    def _min_length(self, length: int):
        self.configuration.metadata.min_length = length
        return self

    def _max_length(self, length: int):
        self.configuration.metadata.max_length = length
        return self

    def _length(self, length: int):
        self.configuration.length = length
        return self

    def _decimal_precision(self, precision: int):
        self.configuration.metadata.decimal_precision = precision
        return self

    def _decimal_scale(self, scale: int):
        self.configuration.metadata.decimal_scale = scale
        self.configuration.scale = scale
        return self

    def _unicode(self, is_unicode: bool = True):
        self.configuration.app_data["is_unicode_string"] = is_unicode
        self.configuration.unicode = FIELD.Unicode.true if is_unicode else FIELD.Unicode.false
        return self

    def _unsigned(self):
        self.configuration.metadata.is_signed = False
        self.configuration.signed = False
        return self

    def _extended_type(self, typ: str):
        self.configuration.app_data["extended_type"] = typ
        self.configuration.extended_type = typ
        self._decimal_scale(6)  # default changes to 6, not sure why
        return self

    def _cluster_key_change(self):
        self.configuration.app_data["cluster_key_change"] = True
        return self

    def _key_change(self):
        self.configuration.app_data["key_change"] = True
        return self

    def difference(self):
        self.configuration.app_data["difference"] = True
        return self

    def _dimension_min_size(self, size: int)-> T:
        self.configuration.app_data["dimension_min_size"] = size
        self.configuration.dimension_min_size = size
        return self

    def _dimension_max_size(self, size: int)-> T:
        self.configuration.app_data["dimension_max_size"] = size
        # FYI: dimension_max_size is not a model property, while dimension_min_size is
        return self

    def _time_scale(self, time_scale: int)-> T:
        self.configuration.app_data["time_scale"] = time_scale
        self.configuration.time_scale = time_scale
        return self

    def _c_format(self, c_format: str)-> T:
        self.configuration.c_format = c_format
        return self

    def _default(self, default: str | int)-> T:
        self.configuration.default = default
        return self

    def _date_format(self, date_format: str)-> T:
        self.configuration.date_format = date_format
        return self

    def _decimal_separator(self, decimal_separator: FIELD.DecimalSeparator)-> T:
        self.configuration.decimal_separator = decimal_separator
        return self

    def _timestamp_format(self, timestamp_format: str)-> T:
        self.configuration.timestamp_format = timestamp_format
        return self

    def _out_format(self, out_format: str)-> T:
        self.configuration.out_format = out_format
        return self

    def _link_field_reference(self, link_field_reference: str)-> T:
        self.configuration.reference = link_field_reference
        return self

    def _padchar(self, padchar: FIELD.PadChar)-> T:
        self.configuration.padchar = padchar
        return self

    def _prefix_bytes(self, prefix_bytes: FIELD.Prefix)-> T:
        self.configuration.prefix_bytes = prefix_bytes
        return self

    def vector_prefix(self, vector_prefix: FIELD.VectorPrefix)-> T:
        self.configuration.vector_prefix = vector_prefix
        return self

    def _epoch(self, epoch: int)-> T:
        self.configuration.epoch = epoch
        return self

    def _max_width(self, max_width: int)-> T:
        self.configuration.max_width = max_width
        return self

    def _precision(self, precision: int)-> T:
        self.configuration.precision = precision
        return self

    def _time_scale_factor(self, time_scale_factor: int)-> T:
        self.configuration.time_scale_factor = time_scale_factor
        return self

    def _start_position(self, start_position: int)-> T:
        self.configuration.start_position = start_position
        return self

    def _generated_percent_invalid(self, generated_percent_invalid: int)-> T:
        self.configuration.generated_percent_invalid = generated_percent_invalid
        return self

    def _generated_percent_zeros(self, generated_percent_zeros: int)-> T:
        self.configuration.generated_percent_zeros = generated_percent_zeros
        return self

    def _tagcase(self, tagcase: int)-> T:
        self.configuration.tagcase = tagcase
        return self

    def _days_since(self, days_since: int)-> T:
        self.configuration.days_since = days_since
        return self

    def _width(self, width: int)-> T:
        self.configuration.width = width
        return self

    def _generate_on_output(self)-> T:
        self.configuration.generate = True
        return self

    def _julian(self)-> T:
        self.configuration.julian = True
        return self

    def _link_keep(self)-> T:
        self.configuration.link_keep = True
        return self

    def _check_decimal_packed(self)-> T:
        self.configuration.check_decimal_packed = True
        return self

    def _export_ebcdic_as_ascii(self)-> T:
        self.configuration.export_ebcdic_as_ascii = True
        return self

    def _decimal_packed_signed(self)-> T:
        self.configuration.decimal_packed_signed = True
        return self

    def _midnight_seconds(self)-> T:
        self.configuration.midnight_seconds = True
        return self

    def _decimal_packed(self, packed_option: FIELD.DecimalPacked)-> T:
        self.configuration.decimal_packed = packed_option
        return self

    def _byte_order(self, byte_order: FIELD.ByteOrder)-> T:
        self.configuration.byte_order = byte_order
        return self

    def _charset(self, charset: FIELD.CharSet)-> T:
        self.configuration.charset = charset
        return self

    def _allow_all_zeros(self)-> T:
        self.configuration.allow_all_zeros = FIELD.AllowAllZeros.fix_zero
        return self

    def _sign_position(self, sign_position: FIELD.SignPosition)-> T:
        self.configuration.sign_position = sign_position
        return self

    def _data_format(self, data_format: FIELD.DataFormat)-> T:
        self.configuration.data_format = data_format
        return self

    def level_number(self, level_number: int)-> T:
        self.configuration.level_no = level_number
        self.configuration.metadata.item_index = level_number
        return self

    def vector(self, vector_length_type: FIELD.VectorType):
        instance_to_update = self
        if vector_length_type == FIELD.VectorType.variable:
            instance_to_update = self._dimension_min_size(0)
        instance_to_update.configuration.vector_length_type = vector_length_type
        return self

    def vector_occurs(self, vector_occurs: int)-> T:
        self.configuration.occurs = vector_occurs
        self._dimension_min_size(vector_occurs)
        self._dimension_max_size(vector_occurs)
        return self

    def _byte_to_skip(self, byte_to_skip: int)-> T:
        self.configuration.byte_to_skip = byte_to_skip
        return self

    def _delim(self, delim: FIELD.Delim)-> T:
        self.configuration.delim = delim
        return self

    def _delim_string(self, delim_string: str)-> T:
        self.configuration.delim_string = delim_string
        return self

    def _quote(self, quote: FIELD.Quote)-> T:
        self.configuration.quote = quote
        return self

    def _cycle_increment(self, cycle_increment: FIELD.CycleIncrement | int)-> T:
        self.configuration.cycle_increment = cycle_increment
        return self

    def _cycle_initial_value(self, cycle_initial_value: FIELD.CycleInitialValue | int)-> T:
        self.configuration.cycle_initial_value = cycle_initial_value
        return self

    def _cycle_limit(self, cycle_limit: FIELD.CycleLimit | int)-> T:
        self.configuration.cycle_limit = cycle_limit
        return self

    def _cycle_values(self, cycle_values: list[int | str])-> T:
        self.configuration.cycle_values = cycle_values
        return self

    def _alphabet(self, alphabet: str)-> T:
        self.configuration.alphabet = alphabet
        return self

    def _generate_algorithm(self, generate_algorithm: FIELD.GenerateAlgorithm)-> T:
        self.configuration.generate_algorithm = generate_algorithm
        return self

    def _generate_type(self, generate_type: FIELD.GenerateType)-> T:
        self.configuration.generate_type = generate_type
        return self

    def _random_limit(self, random_limit: FIELD.RandomLimit)-> T:
        self.configuration.random_limit = random_limit
        return self

    def _random_seed(self, random_seed: FIELD.RandomSeed)-> T:
        self.configuration.random_seed = random_seed
        return self

    def _random_signed(self)-> T:
        self.configuration.random_signed = True
        return self

    def _use_current_date(self)-> T:
        self.configuration.use_current_date = True
        return self

    def _round(self, rounding: FIELD.Round)-> T:
        self.configuration.rounding = rounding
        return self

    def _increment_scale(self, increment_scale: int)-> T:
        self.configuration.increment_scale = increment_scale
        return self

    def actual_field_length(self, length: int):
        self.configuration.actual_length = length
        return self

    def null_field_length(self, length: int):
        self.configuration.null_length = length
        return self

    def null_field_value(self, value: str):
        self.configuration.null_field = value
        return self

    # It is possible this option is not meant to be settable and the canvas option is a bug
    def _decimal_type_scale(self, scale: int):
        self.configuration.increment_scale = scale
        return self

    def _null_seed(self, seed: int):
        self.configuration.nullseed = seed
        return self

    def _percent_null(self, percent_null: int):
        self.configuration.nulls = percent_null
        return self

    def change_code(self):
        self.configuration.app_data["change_code"] = True
        return self

    def derivation(self, derivation: str):
        self.configuration.app_data["derivation"] = derivation
        return self



from collections import deque
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.services.datastage.codegen.code_generator import MasterCodeGenerator
    from ibm_watsonx_data_integration.services.datastage.models.flow.dag import DAG, Link, Node


def format_dag(master_gen: "MasterCodeGenerator", dag: "DAG", schema: dict, composer: str = "flow") -> str:
    visited = set()

    roots = [node for node in dag.nodes() if not node.metadata["parents"]]
    q = deque(roots)

    lines: list[str] = []
    while q:
        node = q.popleft()
        if node in visited:
            continue
        visited.add(node)

        for child in node.metadata["children"]:
            for link in node.get_links_to(child):
                lines.append(format_connect_output_to(master_gen, node, child, link))
                schema_name, schema_code = None, None
                if link in schema:
                    schema_name, schema_code = schema[link]
                lines.append(format_link_configuration(master_gen, link, schema_name, schema_code))
            q.appendleft(child)

        lines.append("")

    return "\n".join(lines)


def format_link_settings(master_gen: "MasterCodeGenerator", link: "Link"):
    fmt = ""
    if link.type == "REJECT":
        fmt += ".reject()"
    if link.type == "REFERENCE":
        fmt += ".reference()"
    # if link._schema:
    #     if not (hasattr(link.src, "model") and link.src.model.op_name == "PxCFF"):
    #         fmt += f".use_schema({master_gen.get_object_var(link._schema)})"
    if link.maps_to_link:
        fmt += f".map_to_link({repr(link._maps_to_link)})"
    if link.maps_from_link:
        fmt += f".map_from_link({repr(link._maps_from_link)})"
    return fmt


def format_connect_output_to(master_gen: "MasterCodeGenerator", src: "Node", dest: "Node", link: "Link"):
    if not link.name:
        return ""
    return (
        master_gen.get_object_var(link)
        + " = "
        + master_gen.get_object_var(src)
        + ".connect_output_to("
        + master_gen.get_object_var(dest)
        + ")"
        + format_link_settings(master_gen, link)
    )


def format_link_configuration(master_gen: "MasterCodeGenerator", link: "Link", schema_name: str, schema_code: str):
    ret: list[str] = []
    if link.name:
        ret.append(master_gen.get_object_var(link) + f".name = {repr(link.name)}")
    if link.schema:
        if not (hasattr(link.src, "configuration") and link.src.configuration.op_name == "PxCFF"):
            ret.append(master_gen.get_object_var(link.schema) + " = " + master_gen.get_object_var(link) + ".create_schema()")
            ret.append(schema_code)
    if link.maps_to_link:
        ret.append(master_gen.get_object_var(link) + f".maps_to_link = {repr(link.maps_to_link)}")
    if link.maps_from_link:
        ret.append(master_gen.get_object_var(link) + f".maps_from_link = {repr(link.maps_from_link)}")
    return "\n".join(ret)



# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""This module containing the TrustedProfileAPIClient class."""

import requests
from ibm_watsonx_data_integration.cpd_api.base import BaseAPIClient
from typing import TYPE_CHECKING
from urllib.parse import quote

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.common.auth import BaseAuthenticator

DEFAULT_TRUSTED_PROFILE_API_VERSION = 1


class TrustedProfileAPIClient(BaseAPIClient):
    """The API Client of Trusted Profiles."""

    def __init__(
        self,
        auth: "BaseAuthenticator",  # pragma: allowlist secret
        base_url: str = "https://iam.cloud.ibm.com",
    ) -> None:
        """Initializes the TrustedProfileAPIClient.

        Args:
            auth: The Authentication object.
            base_url: Default is "https://iam.cloud.ibm.com".
        """
        super().__init__(auth=auth, base_url=base_url)
        self.url_path = f"v{DEFAULT_TRUSTED_PROFILE_API_VERSION}/profiles"

    def list_all_trusted_profiles(self, params: dict) -> requests.Response:
        """Lists all trusted profiles under an account.

        Args:
            params: Query parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}"
        response = self.get(url, params)
        return response

    def get_trusted_profile(self, iam_id: str) -> requests.Response:
        """Returns a trusted profile.

        Args:
            iam_id: The IAM ID.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(iam_id, safe='')}"
        response = self.get(url)
        return response



from ibm_watsonx_data_integration.services.datastage.models.enums import SNOWFLAKE_CONNECTION
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestSnowflakeConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "snowflake_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _testtablerandom_1_ = fc.add_stage("Snowflake", "testtablerandom_1")
        _testtablerandom_1_.configuration.ds_enable_quoted_ids = True
        _testtablerandom_1_.configuration.ds_table_name = "TM_DS.testtable-random"
        _testtablerandom_1_.configuration.schema_name = "TM_DS"
        _testtablerandom_1_.configuration.table_name = "testtable-random"

        _testtablerandom_1_.configuration.connection.account_name = "wv03539.us-east-2.aws"
        _testtablerandom_1_.configuration.connection.authentication_method = (
            SNOWFLAKE_CONNECTION.AuthMethod.username_password
        )
        _testtablerandom_1_.configuration.connection.database = "TM_DS_DB"
        # _testtablerandom_1_.configuration.connection.hostname=environ["SNOWFLAKE_HOSTNAME_OR_IP_ADDRESS"]
        _testtablerandom_1_.configuration.connection.password = environ["SNOWFLAKE_PASSWORD"]
        _testtablerandom_1_.configuration.connection.role = "TM_DS_ROLE"
        _testtablerandom_1_.configuration.connection.username = environ["SNOWFLAKE_USERNAME"]
        _testtablerandom_1_.configuration.connection.warehouse = "DS_WH"

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        testtablerandom_1_schema = _testtablerandom_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        testtablerandom_1_schema.add_field("VARCHAR", "COLUMN_1").length(10)
        testtablerandom_1_schema.add_field("DECIMAL", "COLUMN_2").length(38).precision(38)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025

"""This module defines top-level abstractions for WatsonX Data Integration."""

import json
import logging
import requests
from ibm_watsonx_data_integration.common.constants import (
    PROD_BASE_API_URL,
    PROD_BASE_URL,
)
from ibm_watsonx_data_integration.common.exceptions import CloudObjectStorageNotFoundError
from ibm_watsonx_data_integration.common.utils import _get_service_id_to_name_map
from ibm_watsonx_data_integration.cpd_api import (
    AccessGroupsAPIClient,
    AccountAPIClient,
    ConnectionsApiClient,
    GlobalCatalogApiClient,
    GlobalSearchApiClient,
    JobApiClient,
    MeteringApiClient,
    ProjectApiClient,
    ResourceControllerApiClient,
    RoleAPIClient,
    ServiceIDAPIClient,
    TrustedProfileAPIClient,
    UserAPIClient,
)
from ibm_watsonx_data_integration.cpd_models import (
    AccessGroup,
    AccessGroups,
    Account,
    Accounts,
    ConnectionFile,
    ConnectionFiles,
    ConnectionsServiceInfo,
    DatasourceTypes,
    Project,
    Projects,
    Role,
    Roles,
    Service,
    ServiceID,
    ServiceIDs,
    Services,
    TrustedProfile,
    TrustedProfiles,
    UserProfile,
    UserProfiles,
)
from ibm_watsonx_data_integration.services.datastage.api import DatastageFlowApiClient
from ibm_watsonx_data_integration.services.streamsets.api import (
    EngineApiClient,
    EnvironmentApiClient,
    StreamsetsFlowApiClient,
)
from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.common.auth import BaseAuthenticator


logger = logging.getLogger(__name__)


class Platform:
    """Class to interact with IBM Cloud Pak for Data."""

    _account_api: AccountAPIClient
    _resource_controller_api: ResourceControllerApiClient
    _global_catalog_api: GlobalCatalogApiClient
    _global_search_api: GlobalSearchApiClient
    _project_api: ProjectApiClient
    _job_api: JobApiClient
    _environment_api: EnvironmentApiClient
    _streamsets_flow_api: StreamsetsFlowApiClient
    _user_api: UserAPIClient
    _access_group_api: AccessGroupsAPIClient
    _service_id_to_name_map: dict
    _role_api_client: RoleAPIClient
    _current_account: Account | None
    _components_urls: dict

    def __init__(
        self,
        auth: "BaseAuthenticator",  # pragma: allowlist secret
        base_url: str = PROD_BASE_URL,
        base_api_url: str = PROD_BASE_API_URL,
    ) -> None:
        """The __init__ for the Platform class.

        Args:
            auth: The Authentication object.
            base_url: The Cloud Pak for Data URL.
            base_api_url: The Cloud Pak for Data API URL.
        """
        self.base_url = base_url.rstrip("/").replace("https://", "")
        self._components_urls = self._generate_components_urls(base_api_url)
        self._account_api = AccountAPIClient(auth=auth, base_url=self._components_urls["account_management_url"])
        self._resource_controller_api = ResourceControllerApiClient(
            auth=auth, base_url=self._components_urls["resource_controller_url"]
        )
        self._global_catalog_api = GlobalCatalogApiClient(
            auth=auth, base_url=self._components_urls["global_catalog_url"]
        )
        self._global_search_api = GlobalSearchApiClient(auth=auth, base_url=self._components_urls["global_search_url"])
        self._project_api = ProjectApiClient(auth=auth, base_url=self._components_urls["base_api_url"])
        self._job_api = JobApiClient(auth=auth, base_url=self._components_urls["base_api_url"])
        self._environment_api = EnvironmentApiClient(auth=auth, base_url=self._components_urls["base_api_url"])
        self._datastage_flow_api = DatastageFlowApiClient(auth=auth, base_url=self._components_urls["base_api_url"])
        self._streamsets_flow_api = StreamsetsFlowApiClient(auth=auth, base_url=self._components_urls["base_api_url"])
        self._user_api = UserAPIClient(auth=auth, base_url=self._components_urls["user_management_url"])
        self._access_group_api = AccessGroupsAPIClient(auth=auth, base_url=self._components_urls["iam_url"])
        self._service_id_api = ServiceIDAPIClient(auth=auth, base_url=self._components_urls["iam_url"])
        self._trusted_profile_api = TrustedProfileAPIClient(auth=auth, base_url=self._components_urls["iam_url"])
        self._engine_api = EngineApiClient(auth=auth, base_url=self._components_urls["base_api_url"])
        self._connections_api = ConnectionsApiClient(auth=auth, base_url=self._components_urls["base_api_url"])
        self._metering_api = MeteringApiClient(auth=auth, base_url=self._components_urls["base_api_url"])
        self._service_id_to_name_map = _get_service_id_to_name_map(self)
        self._role_api_client = RoleAPIClient(auth=auth, base_url=self._components_urls["iam_url"])

        self._current_account = None

        self.validate()

    def _generate_components_urls(self, base_api_url: str) -> dict:
        """Generates the URLs for the different components of the Platform."""
        return {
            "base_api_url": base_api_url,
            "global_catalog_url": f"https://globalcatalog.{self.base_url}/api",
            "global_search_url": f"https://api.global-search-tagging.{self.base_url}",
            "resource_controller_url": f"https://resource-controller.{self.base_url}",
            "account_management_url": f"https://accounts.{self.base_url}",
            "user_management_url": f"https://user-management.{self.base_url}",
            "iam_url": f"https://iam.{self.base_url}",
        }

    def validate(self) -> None:
        """Perform validation of Platform setup.

        Raises:
            requests.exceptions.ConnectionError: If base_api_url is invalid or dns cannot be resolved.
            ibm_watsonx_data_integration.common.exceptions.IbmCloudApiException: If base_api_url can be resolved
                                                                                        but is incorrect.
        """
        # Check if correct base_api_url provided
        # | Checking only one endpoint to make init as fast as possible.
        # | Also we want to check the url not availability of all endpoints.
        # | raises ConnectionError on DNS resolve problems and IbmCloudApiException on wrong url
        self.connections_service_info

    @property
    def service_instances(self) -> Services:
        """Returns a list of Service Instances.

        Returns:
            An instance of :py:class:`ibm_watsonx_data_integration.cpd_models.Services`
        """
        return Services(self)

    def create_service_instance(
        self,
        instance_type: str,
        name: str,
        target: str | None = None,
        tags: list | None = None,
    ) -> Service:
        """Creates Service Instance.

        Args:
            instance_type: The Instance Type.
            name: The name of the Instance.
            target: The target of the Instance.
            tags: The tags of the Instance.

        Returns:
            A Service instance.
        """
        return Service._create(self, instance_type, name, target, tags)

    def delete_service_instance(self, service: Service, delete_keys: bool = True) -> requests.Response:
        """Delete Resource Instance.

        Args:
            service: The Service to delete.
            delete_keys: Whether to recursively delete resource keys.

        Returns:
            A HTTP response.
        """
        return service._delete(delete_keys=delete_keys)

    @property
    def projects(self) -> Projects:
        """Returns a list of Project objects.

        Returns:
            An instance of :py:class:`ibm_watsonx_data_integration.cpd_models.Projects`
        """
        return Projects(self)

    def create_project(
        self, name: str, description: str = "", tags: list = None, public: bool = False, project_type: str = "wx"
    ) -> Project:
        """Create a Project.

        Args:
            name: The Project name.
            description: The name of the Project.
            tags: The tags of the Project.
            public: Whether the Project is public.
            project_type: Type of the Project - 'cpd' - IBM Cloud Pak for Data; 'wx' - IBM watsonx (and Data Fabric).

        Returns:
            A project instance.
        """
        return Project._create(self, name, description=description, tags=tags, public=public, project_type=project_type)

    def _get_cloud_storage(self) -> list:
        search_response = self._global_search_api.get_resources(
            json.dumps({"query": "region:global AND service_name:cloud-object-storage", "fields": ["*"]})
        ).json()
        items = search_response.get("items", [])
        if not items:
            raise CloudObjectStorageNotFoundError("Cloud Object Storage does not exist. Cannot proceed.")

        return items[0]["doc"]

    def delete_project(self, project: Project) -> requests.Response:
        """Deletes a Project.

        Args:
            project: The Project to delete.

        Returns:
            A HTTP response.
        """
        return project._delete()

    def update_project(self, project: Project) -> requests.Response:
        """Updates a Project.

        Args:
            project: The Project to delete.

        Returns:
            A HTTP response.
        """
        return project._update()

    @property
    def accounts(self) -> Accounts:
        """Returns a list of all accounts associated with the current IAM identity or API key.

        Returns:
            A list of account objects retrieved from the API.
        """
        return Accounts(self)

    @property
    def current_account(self) -> Account:
        """Gets the current (first) account from the list of accounts.

        Returns:
            An account object representing the first account retrieved from the accounts list.

        Raises:
            ValueError: If there are no accounts available for the provided IAM identity or API key.
        """
        if not self._current_account:
            all_accounts = self.accounts
            if not all_accounts:
                raise ValueError("No accounts available for the provided IAM identity or API key.")
            self._current_account = all_accounts[0]
        return self._current_account

    @current_account.setter
    def current_account(self, account: Account) -> None:
        """Override which account will be used for all subsequent calls.

        Args:
            account: The Account object to set as the current account.

        Returns:
            None.
        """
        self._current_account = account

    @property
    def users(self) -> UserProfiles:
        """Retrieves collection of all the user profiles in the current account.

        Returns:
            An iterable collection of user profiles.
        """
        return UserProfiles(self)

    @property
    def roles(self) -> Roles:
        """Returns a list of all roles.

        Returns:
            An instance of :py:class:`watsonx_di_sdk.role_model.Roles`
        """
        return Roles(self)

    def create_role(
        self, name: str, display_name: str, service_name: str, actions: list, description: str | None = None
    ) -> Role:
        """Creates a custom role for a specific service within the account.

        Args:
            display_name: The display the name of the role that is shown in the console.
            actions: The actions of the role (list of strings).
            name: The name of the role that is used in the CRN. This must be alphanumeric and capitalized.
            service_name: The service name.
            description: The description of the role.

        Returns:
            A role instance.
        """
        return Role._create(self, name, display_name, service_name, actions, description)

    def update_role(self, role: Role) -> requests.Response:
        """Update a custom role.

        A role administrator might want to update an existing custom role by updating the display name, description, or
        the actions that are mapped to the role. The name, account_id, and service_name can't be changed.

        Args:
            role: The role to update.

        Returns:
            A HTTP response.

        Raises:
            TypeError: If you try to modify service_role or system_role.
        """
        return role._update()

    def delete_role(self, role: Role) -> requests.Response:
        """Delete a custom role.

        Args:
            role: The role to delete.

        Returns:
            A HTTP response.

        Raises:
            TypeError: If you try to delete service_role or system_role.
        """
        return role._delete()

    @property
    def trusted_profiles(self) -> TrustedProfiles:
        """Returns a collection of TrustedProfile objects.

        Returns:
            A collection of TrsutedProfile objects retrieved from the API.
        """
        return TrustedProfiles(self)

    @property
    def service_ids(self) -> ServiceIDs:
        """Returns a collection of ServiceID objects.

        Returns:
            A collection of ServiceID objects retrieved from the API.
        """
        return ServiceIDs(self)

    @property
    def access_groups(self) -> AccessGroups:
        """Returns a collection of AccessGroup objects.

        Returns:
            A collection of Access Group objects retrieved from the API.
        """
        return AccessGroups(self)

    def create_access_group(self, name: str, description: str | None = None) -> AccessGroup:
        """Creates an access group and returns the created access group object.

        Args:
            name: Name of the Access Group
            description: Description of the Access Group

        Returns:
            An Access Group instance.
        """
        return AccessGroup._create(self, name, description)

    def update_access_group(self, access_group: AccessGroup) -> requests.Response:
        """Updates an existing group and returns the output for api call to update.

        Args:
            access_group: Access Group to be updated

        Returns:
            A HTTP response.
        """
        return access_group._update()

    def delete_access_group(self, access_group: AccessGroup) -> requests.Response:
        """Deletes an access group and returns output of api call to delete.

        Args:
            access_group: Access Group to be deleted

        Returns:
            A HTTP response.
        """
        return access_group._delete()

    def add_member_to_multiple_access_groups(
        self, member: UserProfile | ServiceID | TrustedProfile, access_groups: list[AccessGroup]
    ) -> requests.Response:
        """Adds a member to multiple Access Groups.

        Args:
            member: The member object to add.
            access_groups: The list of access groups for member to be added to.

        Returns:
            A HTTP response.
        """
        list_of_ag_ids = []

        for ag in access_groups:
            list_of_ag_ids.append(ag.id)

        data = {"type": member.type, "groups": list_of_ag_ids}
        data = json.dumps(data)

        params = {"account_id": self._current_account.account_id}

        return self._access_group_api.add_member_to_multiple_access_groups(
            params=params, iam_id=member.iam_id, data=data
        )

    def remove_member_from_all_access_groups(
        self, member: UserProfile | ServiceID | TrustedProfile
    ) -> requests.Response:
        """Removes a member from all access groups under an account.

        Args:
            member: The member object to remove.

        Returns:
            A HTTP response.
        """
        params = {"account_id": self._current_account.account_id}

        return self._access_group_api.remove_member_from_all_access_groups(iam_id=member.iam_id, params=params)

    @property
    def datasources(self) -> DatasourceTypes:
        """Retrieves available datasource types.

        Returns:
            A list of available DatasourceTypes.
        """
        return DatasourceTypes(platform=self)

    @property
    def connections_service_info(self) -> ConnectionsServiceInfo:
        """Retrieves information about connection service. Can be used as heartheat mechanism.

        Returns:
            A connection service information.
        """
        version = self._connections_api.get_version(params=dict())
        return ConnectionsServiceInfo(**version.json())

    @property
    def files(self) -> ConnectionFiles:
        """Retrieves list of files.

        Returns:
            List of connection files.
        """
        return ConnectionFiles(platform=self)

    def upload_file(self, name: str, file: Path) -> requests.Response:
        """Uploads file.

        Args:
            name: Name of the file to upload.
            file: File to upload.

        Returns:
            A HTTP response.

        Raises:
            ValueError: If incorrect file path is provided.
            FileNotFoundError: If file path does not exist.
        """
        if not file.is_file():
            raise ValueError("Incorrect file path provided.")
        if not file.exists():
            raise FileNotFoundError("File path does not exist.")

        return self._connections_api.upload_file(file_name=name, file=file)

    def delete_file(self, file: ConnectionFile) -> requests.Response:
        """Delete a file.

        Returns:
            A HTTP response.
        """
        return file._delete()

    @property
    def available_engine_versions(self) -> list[str]:
        """Lists all StreamSets Engine Versions.

        Returns:
            A list of Engine versions.
        """
        response = self._environment_api.get_engine_versions()
        return [data["engine_version_id"] for data in response.json()["streamsets_engine_versions"]]

    def get_engine_version_info(self, engine_version: str) -> dict:
        """Get a StreamSets Engine information.

        Returns:
            A dict with information about engine version.

        Raises:
            HTTPError: If the request fails.
        """
        try:
            return self._environment_api.get_engine_by_version(engine_version=engine_version).json()
        except requests.HTTPError as e:
            if e.response.status_code == 404:
                logger.warning(
                    "Engine version: %s does not exist. "
                    "Please check available engine versions using platform.available_engine_versions property.",
                    engine_version,
                )
            raise



from ibm_watsonx_data_integration.services.datastage.models.enums import AMAZON_REDSHIFT_CONNECTION
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAmazonRedShiftConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "amazonredshift_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _autotest_1_ = fc.add_stage("Amazon Redshift", "autotest_1")
        _autotest_1_.configuration.schema_name = "testredshift"
        _autotest_1_.configuration.table_name = "autotest"

        _autotest_1_.configuration.connection.database = "dev"
        _autotest_1_.configuration.connection.hostname_or_ip_address = environ["AMAZONREDSHIFT_HOST"]
        _autotest_1_.configuration.connection.password = environ["AMAZONREDSHIFT_PASSWORD"]
        _autotest_1_.configuration.connection.port = "5439"
        _autotest_1_.configuration.connection.port_is_ssl_enabled = True
        _autotest_1_.configuration.connection.time_type = AMAZON_REDSHIFT_CONNECTION.TimeType.timestamp
        _autotest_1_.configuration.connection.username = environ["AMAZONREDSHIFT_USERNAME"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        autotest_1_schema = _autotest_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        autotest_1_schema.add_field("CHAR", "col1").nullable().length(6)
        autotest_1_schema.add_field("CHAR", "col2").nullable().length(6)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from abc import ABC, abstractmethod
from ibm_watsonx_data_integration.services.datastage.models.flow.dag import Node


class BoundingBox(ABC):
    def __init__(self, *, width: float = 0, height: float = 0, x: float = 0, y: float = 0):
        # POSITIONING IS ALWAYS RELATIVE TO PARENT
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.x_velocity = 0.0
        self.y_velocity = 0.0
        self.x_force = 0.0
        self.y_force = 0.0

    @abstractmethod
    def update_absolute_positions(self, parent_x: float, parent_y: float):
        pass

    def center_x(self):
        return self.x + self.width / 2

    def center_y(self):
        return self.y + self.height / 2

    def y_boundary_distance(self, other: "BoundingBox"):
        return min(
            abs(self.y - (other.y + other.height)),
            abs(other.y - (self.y + self.height)),
        )

    def get_x_from_center(self, center_x: float):
        return center_x - self.width / 2

    def get_y_from_center(self, center_y: float):
        return center_y - self.height / 2

    def add_force(self, x_force: float, y_force: float):
        self.x_force += x_force
        self.y_force += y_force

    def apply_forces(self):
        self.x_velocity += self.__clamp_magnitude(self.x_force, 60)
        self.y_velocity += self.__clamp_magnitude(self.y_force, 60)
        self.x_velocity *= 0.5
        self.y_velocity *= 0.5
        self.x_force = 0
        self.y_force = 0
        self.x += self.x_velocity
        self.y += self.y_velocity

    @staticmethod
    def __clamp_magnitude(val: float, magnitude: float):
        if val > 0:
            return min(val, magnitude)
        return max(val, -magnitude)


class NodeBoundingBox(BoundingBox):
    def __init__(self, node: Node, width: float = 0, height: float = 0):
        super().__init__(width=width, height=height)
        self.node = node

    def update_absolute_positions(self, parent_x: float, parent_y: float):
        self.node.metadata["x"] = round(parent_x + self.x)
        self.node.metadata["y"] = round(parent_y + self.y)
        self.node.metadata["width"] = round(self.width)
        self.node.metadata["height"] = round(self.height)


class GroupBoundingBox(BoundingBox):
    def __init__(self, *, width: float = 0, height: float = 0, x: float = 0, y: float = 0):
        super().__init__(width=width, height=height, x=x, y=y)
        self.children: list[BoundingBox] = []

    def center_within(self, other: BoundingBox):
        self.x = other.center_x() - self.width / 2
        self.y = other.center_y() - self.height / 2

    def update_absolute_positions(self, parent_x: float, parent_y: float):
        for child in self.children:
            child.update_absolute_positions(parent_x + self.x, parent_y + self.y)

    def arrange_children_vertically(self, spacing: float = 0):
        """Arrange children vertically with the specified spacing."""
        y = self.y
        for child in self.children:
            child.y = y
            y += child.height + spacing
        self.height = y - self.y

    def arrange_children_horizontally(self, spacing: float = 0):
        """Arrange children horizontally with the specified spacing."""
        x = self.x
        for child in self.children:
            child.x = x
            x += child.width + spacing
        self.width = x - self.x

    def round(self):
        self.x = round(self.x)
        self.y = round(self.y)
        self.width = round(self.width)
        self.height = round(self.height)



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025

"""Module containing the Connections API client."""

import requests
from .base import BaseAPIClient
from ibm_watsonx_data_integration.common.auth import BaseAuthenticator
from ibm_watsonx_data_integration.cpd_api.adapters import DefaultHTTPAdapter
from pathlib import Path
from typing import Any
from urllib.parse import quote

DEFAULT_CONNECTIONS_API_VERSION = 2
DEFAULT_DATASOURCE_TYPES_API_VERSION = 2


class ConnectionsApiClient(BaseAPIClient):
    """The API client for resources related with Connections."""

    def __init__(self, auth: BaseAuthenticator, base_url: str = "https://api.dataplatform.cloud.ibm.com") -> None:
        """The __init__ of the ConnectionsApiClient.

        Args:
            auth: The Authentication object.
            base_url: The Cloud Pak for Data URL. Default: ``https://api.dataplatform.cloud.ibm.com``.
        """
        super().__init__(auth=auth, base_url=base_url)
        self.url_path_connections = f"v{DEFAULT_CONNECTIONS_API_VERSION}/connections"
        self.url_path_datasource_types = f"v{DEFAULT_DATASOURCE_TYPES_API_VERSION}/datasource_types"

    def get_connections(self, params: dict[str, Any]) -> requests.Response:
        """List defined connections.

        Args:
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}"
        return self.get(url=url, params=params)

    def add_connection(self, data: dict[str, Any], params: dict[str, Any]) -> requests.Response:
        """Define connection.

        Args:
            params: REST Query Parameters.
            data: REST Query payload. This should be an entity (without `metadata`) of the connection.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}"
        return self.post(url=url, params=params, data=data)

    def get_connection(self, connection_id: str, params: dict[str, Any]) -> requests.Response:
        """Get connection.

        Args:
            connection_id: Connection id.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/{quote(connection_id, safe='')}"
        return self.get(url=url, params=params)

    def delete_connection(self, connection_id: str, params: dict[str, Any]) -> requests.Response:
        """Delete connection.

        Args:
            connection_id: Connection id.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/{quote(connection_id, safe='')}"
        return self.delete(url=url, params=params)

    def update_connection(self, connection_id: str, data: dict[str, Any], params: dict[str, Any]) -> requests.Response:
        """Update connection.

        Args:
            connection_id: Connection id.
            data: REST Query payload.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/{quote(connection_id, safe='')}"
        adapter = DefaultHTTPAdapter(auth=self._auth)
        adapter._custom_headers["Content-Type"] = "application/json-patch+json;charset=utf-8"

        return self.patch(url=url, params=params, data=data, adapter=adapter)

    def copy_connection(self, connection_id: str, params: dict[str, Any]) -> requests.Response:
        """Copy connection.

        Args:
            connection_id: Connection id.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/{quote(connection_id, safe='')}/copy"
        return self.post(url=url, params=params)

    def get_connections_assets(self, data: dict[str, Any], params: dict[str, Any]) -> requests.Response:
        """Discover connections assets.

        Args:
            data: REST Query payload. This should be an entity (without `metadata`) of the connection.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/assets"
        return self.post(url=url, params=params, data=data)

    def get_connection_assets(self, connection_id: str, params: dict[str, Any]) -> requests.Response:
        """Get connection assets.

        Args:
            connection_id: Connection id.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/{quote(connection_id, safe='')}/assets"
        return self.get(url=url, params=params)

    def get_connections_asset_data(self, connection_asset_id: str, params: dict[str, Any]) -> requests.Response:
        """Get connections asset data.

        Args:
            connection_asset_id: Connection asset id.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/assets/{quote(connection_asset_id, safe='')}"
        return self.get(url=url, params=params)

    def get_connection_actions(self, connection_id: str, params: dict[str, Any]) -> requests.Response:
        """Get connection actions.

        Args:
            connection_id: Connection id.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/{quote(connection_id, safe='')}/actions"
        return self.get(url=url, params=params)

    def perform_connection_action(
        self, connection_id: str, action_name: str, params: dict[str, Any], data: dict[str, Any]
    ) -> requests.Response:
        """Perform connection action.

        Args:
            connection_id: Connection id.
            action_name: Action name.
            params: REST Query Parameters.
            data: Action request configuration.

        Returns:
            A HTTP response.
        """
        url = (
            f"{self.base_url}/{self.url_path_connections}/{quote(connection_id, safe='')}/actions/"
            f"{quote(action_name, safe='')}"
        )
        return self.put(url=url, params=params, data=data)

    def get_version(self, params: dict[str, Any] | None = None) -> requests.Response:
        """Gets version and other information about the connections service.

        Args:
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/version"
        return self.get(url=url, params=params or dict())

    def get_functional_ids(self, params: dict[str, Any] | None = None) -> requests.Response:
        """Get the list of functional IDs.

        Args:
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/functional_ids"
        return self.get(url=url, params=params or dict())

    def get_file(self, name: str, hash: str, output: Path) -> None:
        """Download a file.

        Args:
            name: The name of file to download.
            hash: The hash of file to download.
            output: Output path for the downloading file.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/files/{quote(name, safe='')}"

        adapter = DefaultHTTPAdapter(auth=self._auth)
        adapter._custom_headers["Content-Type"] = "application/octet-stream"

        params = {"hash": hash}
        response = self.get(url=url, params=params, adapter=adapter)
        with output.open("wb") as byte_file:
            byte_file.write(response.content)

    def upload_file(self, file_name: str, file: Path) -> requests.Response:
        """Upload a file and get back a signed handle in 'Location' header with a hash.

        Args:
            file_name: The name of file to upload.
            file: Path to the uploaded file.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/files/{quote(file_name, safe='')}"

        adapter = DefaultHTTPAdapter(auth=self._auth)
        adapter._custom_headers["Content-Type"] = "application/octet-stream"

        with file.open("rb") as byte_file:
            return self.post(url=url, data=byte_file.read(), adapter=adapter)

    def delete_file(self, name: str, hash: str) -> requests.Response:
        """Delete a file.

        Args:
            name: The name of file to delete.
            hash: The hash of file to download.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/files/{quote(name, safe='')}"
        params = {"hash": hash}
        return self.delete(url=url, params=params)

    def get_file_list(self, params: dict[str, Any] | None = None) -> requests.Response:
        """List files available in mounted directory.

        Args:
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/files"
        return self.get(url=url, params=params or dict())

    def move_files(self, params: dict[str, Any]) -> requests.Response:
        """Migrates files from old locations to new one.

        Args:
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/files"
        return self.put(url=url, params=params)

    def get_datasources(self, params: dict[str, Any] | None = None) -> requests.Response:
        """Gets all defined types of data sources.

        Args:
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        params = params or dict()
        params["connection_properties"] = True
        url = f"{self.base_url}/{self.url_path_datasource_types}"
        return self.get(url=url, params=params)

    def add_datasource(self, data: dict[str, Any]) -> requests.Response:
        """Defines a data source type.

        Args:
            data: REST Query Payload. The definition of the data source type.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_datasource_types}"
        return self.post(url=url, data=data)

    def get_datasource(self, datasource_type: str, params: dict[str, Any]) -> requests.Response:
        """Get details for type of data source.

        Args:
            datasource_type: The data source type.
            params: REST Query Parameters.

        Returns:
            A HTTP response.
        """
        params = params or dict()
        params["connection_properties"] = True
        url = f"{self.base_url}/{self.url_path_datasource_types}/{quote(datasource_type, safe='')}"
        return self.get(url=url, params=params)

    def delete_datasource(self, datasource_type: str) -> requests.Response:
        """Deletes a data source type definition.

        Args:
            datasource_type: The data source type.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_datasource_types}/{quote(datasource_type, safe='')}"
        return self.delete(url=url)

    def update_datasource(self, datasource_type: str) -> requests.Response:
        """Updates the definition of a datasource type.

        Args:
            datasource_type: The data source type.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_datasource_types}/{quote(datasource_type, safe='')}"
        return self.patch(url=url)

    def get_swagger(self) -> requests.Response:
        """Retrieve the swagger definitions to retrieve connections.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path_connections}/api/docs"
        return self.get(url=url)



import ibm_watsonx_data_integration.services.datastage.models.flow_json_model as models
from ibm_watsonx_data_integration.services.datastage.models.enums import FIELD
from ibm_watsonx_data_integration.services.datastage.models.schema.field.base_field import BaseField
from typing import TypeVar
T = TypeVar("T", bound="NVarChar")

class NVarChar(BaseField):
    def __init__(self, name: str):
        super().__init__(name)
        self._set_model_type("string")
        self._set_app_type_code("STRING")
        self._set_app_odbc_type("WVARCHAR")
        self.configuration.metadata.min_length = 0
        self.configuration.metadata.max_length = 100

    def _create_from_model(self, model: models.FieldModelComplex)-> T:
        field = NVarChar(model.name)
        field.configuration = model
        return field

    def length(self, length: int):
        """Set the length of this field.

        Args:
            length: The length to set for this field.

        Returns:
            A new instance of the field with the updated length.

        """
        return self._length(length)._max_length(length)

    def byte_to_skip(self, num_bytes: int):
        return self._byte_to_skip(num_bytes)

    def delimiter(self, delim: FIELD.Delim):
        return self._delim(delim)

    def delimiter_string(self, delimiter_string: str):
        return self._delim_string(delimiter_string)

    def generate_on_output(self):
        return self._generate_on_output()

    def prefix_bytes(self, prefix_bytes: FIELD.Prefix):
        return self._prefix_bytes(prefix_bytes)

    def quote(self, quote_type: FIELD.Quote):
        return self._quote(quote_type)

    def start_position(self, position: int):
        return self._start_position(position)

    def tag_case_value(self, tag_case_value: int):
        return self._tagcase(tag_case_value)

    def default(self, default: str):
        return self._default(default)

    def field_max_width(self, max_width: int):
        return self._max_width(max_width)

    def field_width(self, width: int):
        return self._width(width)

    def is_link_field(self):
        return self._link_keep()

    def padchar(self, padchar: FIELD.PadChar):
        return self._padchar(padchar)

    def generate_algorithm(self, generate_algorithm: FIELD.GenerateAlgorithm):
        return self._generate_algorithm(generate_algorithm)

    def cycle_values(self, cycle_values: list[str]):
        return self.cycle_values(cycle_values)

    def alphabet(self, alphabet: str):
        return self._alphabet(alphabet)

    def link_field_reference(self, link_field_reference):
        return self._link_field_reference(link_field_reference)

    def null_seed(self, seed: int):
        return self._null_seed(seed)

    def percent_null(self, percent_null: int):
        return self._percent_null(percent_null)



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import SNOWFLAKE_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class AuthMethod(Enum):
    key_pair = "key_pair"
    username_password = "username_password"


class LineageExtractionType(Enum):
    account_usage = "account_usage"
    information_schema = "information_schema"


class SnowflakeConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "2fc1372f-b58c-4d45-b0c4-dfb32a1c78a5"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    account_name: str = Field(None, alias="account_name")
    authentication_method: SNOWFLAKE_CONNECTION.AuthMethod | None = Field(
        SNOWFLAKE_CONNECTION.AuthMethod.username_password, alias="auth_method"
    )
    url_endpoint: str | None = Field(None, alias="authenticator_url")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    database: str = Field(None, alias="database")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    error_on_nondeterministic_merge: bool | None = Field(None, alias="error_on_nondeterministic_merge")
    hostname_or_ip_address: str | None = Field("@account_name@.snowflakecomputing.com", alias="host")
    keep_client_session_alive: bool | None = Field(None, alias="keep_alive")
    heartbeat_frequency: int | None = Field(None, alias="keep_alive_heartbeat_frequency")
    key_passphrase: str | None = Field(None, alias="key_passphrase")
    lineage_extraction_type: SNOWFLAKE_CONNECTION.LineageExtractionType | None = Field(
        None, alias="lineage_extraction_type"
    )
    password: str = Field(None, alias="password")
    port: int | None = Field(443, alias="port")
    private_key: str = Field(None, alias="private_key")
    query_timeout: int | None = Field(None, alias="query_timeout")
    role: str | None = Field(None, alias="role")
    username: str = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    warehouse: str = Field(None, alias="warehouse")
    properties: str | None = Field(None, alias="properties")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("key_passphrase")
            if (
                (
                    ((not self.defer_credentials))
                    and ((not self.password))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "key_pair"
                                    )
                                    or (self.authentication_method == "key_pair")
                                )
                            )
                        )
                        or (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "key_pair_path"
                                    )
                                    or (self.authentication_method == "key_pair_path")
                                )
                            )
                        )
                    )
                )
                or (
                    ((not self.defer_credentials))
                    and (not self.password)
                    and (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "key_pair"
                            )
                            or (self.authentication_method == "key_pair")
                        )
                    )
                )
            )
            else exclude.add("key_passphrase")
        )
        (
            include.add("private_key")
            if (
                ((not self.defer_credentials))
                and (not self.password)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "key_pair"
                        )
                        or (self.authentication_method == "key_pair")
                    )
                )
            )
            else exclude.add("private_key")
        )
        (
            include.add("password")
            if (
                ((not self.defer_credentials))
                and (not self.private_key)
                and (not self.key_passphrase)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "username_password"
                        )
                        or (self.authentication_method == "username_password")
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("username")
            if (
                (
                    ((not self.defer_credentials))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "key_pair"
                                    )
                                    or (self.authentication_method == "key_pair")
                                )
                            )
                        )
                        or (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "key_pair_path"
                                    )
                                    or (self.authentication_method == "key_pair_path")
                                )
                            )
                        )
                        or (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "oauth"
                                    )
                                    or (self.authentication_method == "oauth")
                                )
                            )
                        )
                        or (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "username_password"
                                    )
                                    or (self.authentication_method == "username_password")
                                )
                            )
                        )
                    )
                )
                or (
                    ((not self.defer_credentials))
                    and (
                        (
                            (
                                self.authentication_method
                                and (
                                    (
                                        hasattr(self.authentication_method, "value")
                                        and self.authentication_method.value == "key_pair"
                                    )
                                    or (self.authentication_method == "key_pair")
                                )
                            )
                        )
                        or (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "username_password"
                                )
                                or (self.authentication_method == "username_password")
                            )
                        )
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("key_passphrase")
            if (
                ((not self.defer_credentials))
                and (not self.password)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "key_pair"
                        )
                        or (self.authentication_method == "key_pair")
                    )
                )
            )
            else exclude.add("key_passphrase")
        )
        (
            include.add("username")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "key_pair"
                                )
                                or (self.authentication_method == "key_pair")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "username_password"
                            )
                            or (self.authentication_method == "username_password")
                        )
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")
        include.add("properties") if (self.hidden_dummy_property1) else exclude.add("properties")

        (
            include.add("key_passphrase")
            if (
                (not self.defer_credentials)
                and (not self.password)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "key_pair" in str(self.authentication_method.value)
                        )
                        or ("key_pair" in str(self.authentication_method))
                    )
                    and self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "key_pair_path" in str(self.authentication_method.value)
                        )
                        or ("key_pair_path" in str(self.authentication_method))
                    )
                )
            )
            or (
                (not self.defer_credentials)
                and (not self.password)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "key_pair" in str(self.authentication_method.value)
                        )
                        or ("key_pair" in str(self.authentication_method))
                    )
                )
            )
            else exclude.add("key_passphrase")
        )
        (
            include.add("password")
            if (not self.defer_credentials)
            and (not self.private_key)
            and (not self.key_passphrase)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "username_password" in str(self.authentication_method.value)
                    )
                    or ("username_password" in str(self.authentication_method))
                )
            )
            else exclude.add("password")
        )
        (
            include.add("username")
            if (
                (not self.defer_credentials)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "key_pair" in str(self.authentication_method.value)
                        )
                        or ("key_pair" in str(self.authentication_method))
                    )
                    and self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "key_pair_path" in str(self.authentication_method.value)
                        )
                        or ("key_pair_path" in str(self.authentication_method))
                    )
                    and self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "oauth" in str(self.authentication_method.value)
                        )
                        or ("oauth" in str(self.authentication_method))
                    )
                    and self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "username_password" in str(self.authentication_method.value)
                        )
                        or ("username_password" in str(self.authentication_method))
                    )
                )
            )
            or (
                (not self.defer_credentials)
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "key_pair" in str(self.authentication_method.value)
                        )
                        or ("key_pair" in str(self.authentication_method))
                    )
                    and self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value
                            and "username_password" in str(self.authentication_method.value)
                        )
                        or ("username_password" in str(self.authentication_method))
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("private_key")
            if (not self.defer_credentials)
            and (not self.password)
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "key_pair" in str(self.authentication_method.value)
                    )
                    or ("key_pair" in str(self.authentication_method))
                )
            )
            else exclude.add("private_key")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""This module contains the API for the StreamSets CPD Service."""

from ibm_watsonx_data_integration.services.streamsets.api.engine_api import EngineApiClient
from ibm_watsonx_data_integration.services.streamsets.api.environment_api import EnvironmentApiClient
from ibm_watsonx_data_integration.services.streamsets.api.flow_api import StreamsetsFlowApiClient

__all__ = [
    "StreamsetsFlowApiClient",
    "EnvironmentApiClient",
    "EngineApiClient",
]



from ibm_watsonx_data_integration.services.datastage.models.connections.amazon_postgresql_connection import (
    AmazonPostgresqlConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.amazon_redshift_connection import (
    AmazonRedshiftConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.amazonrds_oracle_connection import (
    AmazonrdsOracleConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.amazons3_connection import Amazons3Conn
from ibm_watsonx_data_integration.services.datastage.models.connections.apache_hbase_connection import ApacheHbaseConn
from ibm_watsonx_data_integration.services.datastage.models.connections.apache_hive_connection import ApacheHiveConn
from ibm_watsonx_data_integration.services.datastage.models.connections.apache_kafka_connection import ApacheKafkaConn
from ibm_watsonx_data_integration.services.datastage.models.connections.azure_blob_storage_connection import (
    AzureBlobStorageConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.azure_cosmos_connection import AzureCosmosConn
from ibm_watsonx_data_integration.services.datastage.models.connections.azure_databricks_connection import (
    AzureDatabricksConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.azure_file_storage_connection import (
    AzureFileStorageConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.azure_postgresql_connection import (
    AzurePostgresqlConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.azuredatalake_connection import (
    AzuredatalakeConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.azuresql_connection import AzuresqlConn
from ibm_watsonx_data_integration.services.datastage.models.connections.azuresynapse_connection import AzuresynapseConn
from ibm_watsonx_data_integration.services.datastage.models.connections.bigquery_connection import BigqueryConn
from ibm_watsonx_data_integration.services.datastage.models.connections.bigsql_connection import BigsqlConn
from ibm_watsonx_data_integration.services.datastage.models.connections.box_connection import BoxConn
from ibm_watsonx_data_integration.services.datastage.models.connections.cassandra_connection import CassandraConn
from ibm_watsonx_data_integration.services.datastage.models.connections.cassandra_datastage_connection import (
    CassandraDatastageConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.cloud_object_storage_connection import (
    CloudObjectStorageConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.cognos_analytics_connection import (
    CognosAnalyticsConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.datastax_connection import DatastaxConn
from ibm_watsonx_data_integration.services.datastage.models.connections.db2_connection import Db2Conn
from ibm_watsonx_data_integration.services.datastage.models.connections.db2cloud_connection import Db2cloudConn
from ibm_watsonx_data_integration.services.datastage.models.connections.db2fordatastage_connection import (
    Db2fordatastageConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.db2iseries_connection import Db2iseriesConn
from ibm_watsonx_data_integration.services.datastage.models.connections.db2warehouse_connection import Db2warehouseConn
from ibm_watsonx_data_integration.services.datastage.models.connections.db2zos_connection import Db2zosConn
from ibm_watsonx_data_integration.services.datastage.models.connections.denodo_connection import DenodoConn
from ibm_watsonx_data_integration.services.datastage.models.connections.derby_connection import DerbyConn
from ibm_watsonx_data_integration.services.datastage.models.connections.dremio_connection import DremioConn
from ibm_watsonx_data_integration.services.datastage.models.connections.dropbox_connection import DropboxConn
from ibm_watsonx_data_integration.services.datastage.models.connections.dv_connection import DvConn
from ibm_watsonx_data_integration.services.datastage.models.connections.dvm_connection import DvmConn
from ibm_watsonx_data_integration.services.datastage.models.connections.elasticsearch_connection import (
    ElasticsearchConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.exasol_connection import ExasolConn
from ibm_watsonx_data_integration.services.datastage.models.connections.ftp_connection import FtpConn
from ibm_watsonx_data_integration.services.datastage.models.connections.generics3_connection import Generics3Conn
from ibm_watsonx_data_integration.services.datastage.models.connections.google_cloud_storage_connection import (
    GoogleCloudStorageConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.google_looker_connection import GoogleLookerConn
from ibm_watsonx_data_integration.services.datastage.models.connections.google_pub_sub_connection import (
    GooglePubSubConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.greenplum_connection import GreenplumConn
from ibm_watsonx_data_integration.services.datastage.models.connections.hdfs_apache_connection import HdfsApacheConn
from ibm_watsonx_data_integration.services.datastage.models.connections.http_connection import HttpConn
from ibm_watsonx_data_integration.services.datastage.models.connections.ibm_mq_connection import IbmMqConn
from ibm_watsonx_data_integration.services.datastage.models.connections.impala_connection import ImpalaConn
from ibm_watsonx_data_integration.services.datastage.models.connections.informix_connection import InformixConn
from ibm_watsonx_data_integration.services.datastage.models.connections.jdbc_connection import JdbcConn
from ibm_watsonx_data_integration.services.datastage.models.connections.mariadb_connection import MariadbConn
from ibm_watsonx_data_integration.services.datastage.models.connections.match360_connection import Match360Conn
from ibm_watsonx_data_integration.services.datastage.models.connections.minio_connection import MinioConn
from ibm_watsonx_data_integration.services.datastage.models.connections.mongodb_connection import MongodbConn
from ibm_watsonx_data_integration.services.datastage.models.connections.mongodb_ibmcloud_connection import (
    MongodbIbmcloudConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.mysql_amazon_connection import MysqlAmazonConn
from ibm_watsonx_data_integration.services.datastage.models.connections.mysql_compose_connection import MysqlComposeConn
from ibm_watsonx_data_integration.services.datastage.models.connections.mysql_connection import MysqlConn
from ibm_watsonx_data_integration.services.datastage.models.connections.netezza_connection import NetezzaConn
from ibm_watsonx_data_integration.services.datastage.models.connections.netezza_optimized_connection import (
    NetezzaOptimizedConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.odbc_connection import OdbcConn
from ibm_watsonx_data_integration.services.datastage.models.connections.oracle_connection import OracleConn
from ibm_watsonx_data_integration.services.datastage.models.connections.oracle_datastage_connection import (
    OracleDatastageConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.planning_analytics_connection import (
    PlanningAnalyticsConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.postgresql_connection import PostgresqlConn
from ibm_watsonx_data_integration.services.datastage.models.connections.postgresql_ibmcloud_connection import (
    PostgresqlIbmcloudConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.presto_connection import PrestoConn
from ibm_watsonx_data_integration.services.datastage.models.connections.salesforce_connection import SalesforceConn
from ibm_watsonx_data_integration.services.datastage.models.connections.salesforceapi_connection import (
    SalesforceapiConn,
)

# from ibm_watsonx_data_integration.services.datastage.models.connections.sapbapi_connection import SapbapiConn
from ibm_watsonx_data_integration.services.datastage.models.connections.sapbulkextract_connection import (
    SapbulkextractConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.sapdeltaextract_connection import (
    SapdeltaextractConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.saphana_connection import SaphanaConn
from ibm_watsonx_data_integration.services.datastage.models.connections.sapidoc_connection import SapidocConn
from ibm_watsonx_data_integration.services.datastage.models.connections.sapiq_connection import SapiqConn
from ibm_watsonx_data_integration.services.datastage.models.connections.sapodata_connection import SapodataConn
from ibm_watsonx_data_integration.services.datastage.models.connections.singlestore_connection import SinglestoreConn
from ibm_watsonx_data_integration.services.datastage.models.connections.snowflake_connection import SnowflakeConn
from ibm_watsonx_data_integration.services.datastage.models.connections.sqlserver_connection import SqlserverConn
from ibm_watsonx_data_integration.services.datastage.models.connections.storage_volume_connection import (
    StorageVolumeConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.sybase_connection import SybaseConn
from ibm_watsonx_data_integration.services.datastage.models.connections.tableau_connection import TableauConn
from ibm_watsonx_data_integration.services.datastage.models.connections.teradata_connection import TeradataConn
from ibm_watsonx_data_integration.services.datastage.models.connections.teradata_datastage_connection import (
    TeradataDatastageConn,
)
from ibm_watsonx_data_integration.services.datastage.models.connections.vertica_connection import VerticaConn
from ibm_watsonx_data_integration.services.datastage.models.connections.watsonx_data_connection import WatsonxDataConn


class Connection:
    """Connections are data source connectors which provide data connectivity and metadata integration to external data sources."""

    AmazonRedshift = AmazonRedshiftConn
    Amazons3 = Amazons3Conn
    CassandraDatastage = CassandraDatastageConn
    Cassandra = CassandraConn
    Derby = DerbyConn
    HdfsApache = HdfsApacheConn
    ApacheHive = ApacheHiveConn
    Impala = ImpalaConn
    ApacheKafka = ApacheKafkaConn
    Box = BoxConn
    Datastax = DatastaxConn
    Denodo = DenodoConn
    Dremio = DremioConn
    Dropbox = DropboxConn
    Elasticsearch = ElasticsearchConn
    Odbc = OdbcConn
    Generics3 = Generics3Conn
    Http = HttpConn
    Ftp = FtpConn
    Bigquery = BigqueryConn
    GooglePubSub = GooglePubSubConn
    GoogleCloudStorage = GoogleCloudStorageConn
    GoogleLooker = GoogleLookerConn
    Greenplum = GreenplumConn
    CloudObjectStorage = CloudObjectStorageConn
    CognosAnalytics = CognosAnalyticsConn
    Dv = DvConn
    Dvm = DvmConn
    Bigsql = BigsqlConn
    Db2iseries = Db2iseriesConn
    Db2cloud = Db2cloudConn
    Db2 = Db2Conn
    Db2zos = Db2zosConn
    Db2warehouse = Db2warehouseConn
    Db2fordatastage = Db2fordatastageConn
    Informix = InformixConn
    Match360 = Match360Conn
    IbmMq = IbmMqConn
    NetezzaOptimized = NetezzaOptimizedConn
    Netezza = NetezzaConn
    PlanningAnalytics = PlanningAnalyticsConn
    WatsonxData = WatsonxDataConn
    Mariadb = MariadbConn
    AzureBlobStorage = AzureBlobStorageConn
    AzureCosmos = AzureCosmosConn
    Azuredatalake = AzuredatalakeConn
    AzureDatabricks = AzureDatabricksConn
    AzureFileStorage = AzureFileStorageConn
    Azuresql = AzuresqlConn
    Azuresynapse = AzuresynapseConn
    Sqlserver = SqlserverConn
    MongodbIbmcloud = MongodbIbmcloudConn
    Mongodb = MongodbConn
    MysqlCompose = MysqlComposeConn
    MysqlAmazon = MysqlAmazonConn
    Mysql = MysqlConn
    AmazonrdsOracle = AmazonrdsOracleConn
    Oracle = OracleConn
    OracleDatastage = OracleDatastageConn
    PostgresqlIbmcloud = PostgresqlIbmcloudConn
    AmazonPostgresql = AmazonPostgresqlConn
    AzurePostgresql = AzurePostgresqlConn
    Postgresql = PostgresqlConn
    Presto = PrestoConn
    Salesforce = SalesforceConn
    Salesforceapi = SalesforceapiConn
    Sapodata = SapodataConn
    Sapiq = SapiqConn
    Sybase = SybaseConn
    Singlestore = SinglestoreConn
    Snowflake = SnowflakeConn
    Tableau = TableauConn
    TeradataDatastage = TeradataDatastageConn
    Teradata = TeradataConn
    Vertica = VerticaConn
    ApacheHbase = ApacheHbaseConn
    Exasol = ExasolConn
    Jdbc = JdbcConn
    Minio = MinioConn
    Sapbulkextract = SapbulkextractConn
    Sapdeltaextract = SapdeltaextractConn
    Sapidoc = SapidocConn
    # Sapbapi = SapbapiConn
    Saphana = SaphanaConn
    StorageVolume = StorageVolumeConn



from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestSalesforceConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "salesforce_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _test1_1_ = fc.add_stage("Salesforce.com", "TEST1_1")
        _test1_1_.configuration.schema_name = "SFORCE"
        _test1_1_.configuration.table_name = "TEST1"

        _test1_1_.configuration.connection.password = environ["SALESFORCE_PASSWORD"]
        _test1_1_.configuration.connection.server_name = "login.salesforce.com"
        _test1_1_.configuration.connection.username = environ["SALESFORCE_USERNAME"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        test1_1_schema = _test1_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        test1_1_schema.add_field("VARCHAR", "ID").length(18)
        test1_1_schema.add_field("VARCHAR", "OWNERID").length(18)
        test1_1_schema.add_field("BIT", "ISDELETED")
        test1_1_schema.add_field("VARCHAR", "NAME").nullable().length(80)
        test1_1_schema.add_field("VARCHAR", "CURRENCYISOCODE").nullable().length(3)
        test1_1_schema.add_field("TIMESTAMP", "CREATEDDATE")
        test1_1_schema.add_field("VARCHAR", "CREATEDBYID").length(18)
        test1_1_schema.add_field("TIMESTAMP", "LASTMODIFIEDDATE")
        test1_1_schema.add_field("VARCHAR", "LASTMODIFIEDBYID").length(18)
        test1_1_schema.add_field("TIMESTAMP", "SYSTEMMODSTAMP")
        test1_1_schema.add_field("DATE", "LASTACTIVITYDATE").nullable()
        test1_1_schema.add_field("VARCHAR", "EMPID").length(6)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state
